# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the License.
#
# Authors:
# -----------------------------------------------------------------------------
#   Rueen Fiez <rueenfiez@gmail.com>
#   Vincent Lam <vlam6@learn.senecac.on.ca>
#   Armen Zambrano Gasparnian <armenzg@gmail.com>
# -----------------------------------------------------------------------------
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

# VERSION:
# ==============================
# version 0.5
# ==============================

from optparse import OptionParser
import shutil
import re
import os
import os.path
import Parser
from Parser import DTDParser
from Parser import PropertiesParser

import sys

#try to read a DTD file and attempt to make changes to a "word"
class Tool:
     
     #constructor
     def __init__(self):
          self.data = []



     # DESCRIPTION
     # callback() is passed a string to be translated. It is responsible for changing all parser.contents
     # values. We insert the translation rules in here via regular expressions. This function needs to be
     # optomized since their could be hundreds of translations, thus resulting in a bloated callback() function.
     def callback(self, instring):
          # if we use subn(), which returns a 2-tuple containing
          # the new string value and the number of replacements
          # that were performed
          # DOING so will allow us to know if the file has been modify
          #reg1 = re.compile(r'(.*".*)([Dd])ialogue')
          #reg2 = re.compile(r'(.*".*)([Cc])olor')
          #reg3 = re.compile(r'(.*".*)([Gg])o forward')
          #reg4 = re.compile(r'(.*".*)([Mm])inimize')
          #reg5 = re.compile(r'(.*".*)([Cc])enter')
          #reg6 = re.compile(r'(.*".*)([Oo])rganize')
          #reg7 = re.compile(r'(.*".*)([Cc])ustomize')
          #instring = reg1.sub(r'\1\2ialog', instring)
          #instring = reg2.sub(r'\1\2olour', instring)
          #instring = reg3.sub(r'\1\2o forwards', instring)
          #instring = reg4.sub(r'\1\2inimise', instring)
          #instring = reg5.sub(r'\1\2entre', instring)
          #instring = reg6.sub(r'\1\2rganise', instring)
          #instring = reg7.sub(r'\1\2ustomise', instring)
          
          instring = re.sub(r'(.*)([Cc])olor', r'\1\2olour', instring) # For 0.7 Release: Eventually, this would have to be sub(fromFileValue, fromFileKey). <-- Pseudocode
          
          return instring

     # DESCRIPTION
     # translate() uses the DTDParser and PropertiesParser from the Parser.py class to
     # parse each .dtd and .properties file in the given directory. Its contents
     # are then passed into our callback() function string by string and changed in their.
     def translate(self, node):
          dtdParser = DTDParser()
          propParser = PropertiesParser()
          if(node.endswith('.dtd')):
              f = open(node,'r+')
              dtdParser.read(node)
              f.write((t.callback(dtdParser.contents)).encode("utf-8"))
              # Use for DEBUGGING
              if options.verbose == True:
                   print "DTD_PARSER_CONTENTS: \n" + dtdParser.contents
              f.close()
          elif(node.endswith('.properties')):
              f = open(node,'r+')
              propParser.read(node)
              f.write((t.callback(propParser.contents)).encode("utf-8"))
              # Use for DEBUGGING
              if options.verbose == True:
                   print "PROP_PARSER_CONTENTS: \n" + propParser.contents
              f.close()

     # DESCRIPTION
     # process() is the function responsible for actually "walking" a
     # given directory. It contains the logic used to see if each item in the folder
     # is either a "file" or a "directory".
     def process(self, forkedLocale):
          directories = [forkedLocale]
          while len(directories) > 0: #while there are directories to walk through
              directory = directories.pop() #get one of the directories reachable from CWD
              # Used for DEBUGGING
              if options.absPath == True:
                   print directory
              for name in os.listdir(directory): # we are going to iterate through every element of directory
                  fullpath = os.path.join(directory,name)
                  if os.path.isfile(fullpath):
				      #If is a file, try to make changes ot it
                      # Used for DEBUGGING
                           #print "FULLPATH: " + fullpath
                      t.translate(fullpath)
                  elif os.path.isdir(fullpath): #if it is a dir, let's append it to iterate later on
                      directories.append(fullpath)

     # Will create template.
     def createTemplate(self):
          # The name of the template file --> 'locale_template.properties'
          filename = "locale_template.properties"

          # Check to see if the 'locale_template.properties' file already exists
          if os.path.exists(filename):
               print "The localization template file already exists in your current working directory.\n"
          else:
               # This command creates the file. In later release, not only should it create it, it should also fill it up with Keys.
               FILE = open(filename,'w')
               FILE.close()
               print "A new localization template has been generated in the same location as this script.\n"

     def importTemplate(self):
          print "Hi"

if __name__ == '__main__':

     # Parse command line arguments. Available options below
     parser = OptionParser()

     parser.add_option("-v", "--verbose", action="store_true", dest="verbose", help="Shows details of forking process\n")
     parser.add_option("-a", "--abspath", action="store_true", dest="absPath", help="Shows the absolute path of the files being parsed\n")
     parser.add_option("-t", "--template", action="store_true", dest="template", help="Create a localization template in your cwd\n")
     parser.add_option("-i", "--import", action="store", dest="locale_Template", help="Use this option to import a localization template. To generate one, use the \"-t\" flag\n")
     parser.add_option("-s", "--source", action="store", type="string", dest="source", help="The original source that will be forked\n")
     parser.add_option("-n", "--new", action="store", type="string", dest="newSource", help="The name given to the new forked tree\n")
     (options, args) = parser.parse_args()

     # DEBUG
     #print "Template value: ", options.template

     # Create an instance of the tool
     t = Tool()     # will have to send template String as argument to process() method. [For 0.7 Release]

     # If user wants to generate a template, run the createTemplate() method. If not, it means it's a simple localization command
     if options.template == True:
          t.createTemplate()
     else:
          if (re.search('^\w',options.source) and re.search('^\w', options.newSource)):
               # Copy original tree. The '^' means starts with and the '\w' is short for [a-zA-Z0-9]
               shutil.copytree(options.source, options.newSource)

               # Process changes to new tree
               t.process(options.newSource)  # Validation needed here to see if new forked tree directory exists. Code below for this. [For 0.5 Release]

               # When complete, system returns msg
               print "\nProcessing completed. Check current working directory for " + options.newSource + " source tree.\n"

               # DEBUG
               #print len(options.source)
          else:
               # If in here, it means either the source or newSource arguments were invalid
               print "ERROR: source or new-source name must begin with an alphanumeric character."
     
     
     # ###################################################################################
     # Code below was our original "main" code. Good for quick & dirty testing
     # Version 0.1 - [Deprecated]
     #
     # Original #
     #t = Tool()
     #t.process('en-GB')
     #print raw_input("press Return>")
     #
     # ###################################################################################

     #####################################################################################
     # Version 0.3.2 Main - [Working]
     #
     #t = Tool()
     # Parse command line arguments
     # TODO We could use OptionParser() rather than this way
     #forkedLocale = raw_input("Enter original l10n tree (for now just type 'en')\nPath: ")
     # Some input validation
     #if len(forkedLocale) >= 2: #TODO a regex will make more sense
     #    newLocaleNameFlag = False
     #    while(newLocaleNameFlag == False):
     #         newLocaleName = raw_input("Enter a new locale-regional name (eg; en-CA)\nNew Locale Name: ")
     #         if os.path.exists(newLocaleName) == False:
     #              newLocaleNameFlag = True
     #              shutil.copytree(forkedLocale, newLocaleName)
     #              t.process(newLocaleName)
     #         else:
     #              print "The directory " + newLocaleName + " already exists. Please choose another one.\n"
     #
     #    print raw_input("\nTranslation(s) for " + newLocaleName + " complete. Press ENTER to exit...>")
     #else:
     #    print "\nNeed to input a valid localization path!"
     #
     #####################################################################################

     
     


